# Authors: Stephane Gaiffas
# License: BSD 3 clause

# Use:
# streamlit run kernel-svm.py

import sys
import numpy as np
import matplotlib.pyplot as plt
from matplotlib.colors import ListedColormap

import streamlit as st
from sklearn.datasets import (
    make_moons,
    make_circles,
    make_blobs,
    make_classification,
)
from sklearn.preprocessing import MinMaxScaler

from sklearn.svm import LinearSVC, SVC


def get_mesh(X, h=0.02, padding=0.5):
    """Build a regular meshgrid using the range of the features in X
    """
    x_min, x_max = X[:, 0].min() - padding, X[:, 0].max() + padding
    y_min, y_max = X[:, 1].min() - padding, X[:, 1].max() + padding
    xx, yy = np.meshgrid(np.arange(x_min, x_max, h), np.arange(y_min, y_max, h))
    X_mesh = np.array([xx.ravel(), yy.ravel()]).T
    return xx, yy, X_mesh


def plot_scatter_binary_classif(
    ax,
    xx,
    yy,
    X,
    y,
    s=10,
    alpha=None,
    cm=None,
    title=None,
    fontsize=None,
    lw=None,
    norm=None,
    noaxes=False,
):
    if cm is None:
        cm = plt.get_cmap("RdBu")

    ax.scatter(X[:, 0], X[:, 1], c=y, s=s, cmap=cm, alpha=alpha, lw=lw, norm=norm, 
        label="data")
    if title is not None:
        ax.set_title(title, fontsize=fontsize)
    ax.set_xlim(xx.min(), xx.max())
    ax.set_ylim(yy.min(), yy.max())
    ax.set_xticks(())
    ax.set_yticks(())
    ax.set_xticklabels([])
    ax.set_yticklabels([])
    if noaxes:
        ax.axis("off")


def plot_contour_binary_classif(
    ax, xx, yy, Z, cm=None, alpha=0.8, levels=200, title=None, score=None, norm=None,
    support_vectors=None
):
    if cm is None:
        cm = plt.get_cmap("RdBu")

    cm_bright = ListedColormap(['#FF0000', '#0000FF'])
    ct = ax.contourf(xx, yy, Z, cmap=cm, alpha=alpha, levels=levels, norm=norm)

    ax.set_xlim(xx.min(), xx.max())
    ax.set_ylim(yy.min(), yy.max())
    ax.set_xticks(())
    ax.set_yticks(())
    ax.set_xticklabels([])
    ax.set_yticklabels([])
    if support_vectors is not None:
        ax.scatter(support_vectors[:, 0], support_vectors[:, 1], 
            s=30, cmap=cm_bright, c='green', label='support vectors')
    if title is not None:
        ax.set_title(title)

    plt.colorbar(ct)

    if score is not None:
        ax.text(
            xx.max() - 0.3,
            yy.min() + 0.3,
            ("%.2f" % score).lstrip("0"),
            size=25,
            horizontalalignment="right",
        )


# Some global parameters
n_samples = 101
random_state = 42
# Delta in the meshgrid
h = 0.01
# Number of color levels in the contour plot
levels = 100

st.title("Kernel SVM illustrated")

# The sidebar
st.sidebar.title("Dataset")
st.sidebar.markdown("Choose the dataset below")
dataset = st.sidebar.selectbox(
    "dataset", ["moons", "circles", "linear", "blobs"], index=2
)
show_data = st.sidebar.checkbox("Show data", value=True)
normalize = st.sidebar.checkbox("Normalize colors", value=False)

st.sidebar.title("Parameters")
st.sidebar.markdown("Hyperparameters of the SVM")

kernel = st.sidebar.selectbox(
    "kernel", ["linear", "poly", "rbf", "sigmoid"], index=0
)

if kernel != "linear":
    gamma = st.sidebar.slider("gamma", min_value=.1, max_value=5.0, value=0.5, step=0.1)
else:
    gamma = 0.5

if kernel == "poly":
    degree = st.sidebar.slider("degree", min_value=2, max_value=10, value=2, step=1)
else:
    degree = 2

logC = st.sidebar.slider("log(C)", min_value=-3.0, max_value=3.0, value=0.0, step=0.5)
C = 10 ** logC

if normalize:
    norm = plt.Normalize(vmin=0.0, vmax=1.0)
else:
    norm = None


@st.cache
def simulate_data(dataset="moons"):
    if dataset == "moons":
        X, y = make_moons(n_samples=n_samples, noise=0.2, random_state=random_state)
    elif dataset == "circles":
        X, y = make_circles(
            n_samples=n_samples, noise=0.1, factor=0.5, random_state=random_state
        )
    elif dataset == "linear":
        X, y = make_classification(
            n_samples=n_samples,
            n_features=2,
            n_redundant=0,
            n_informative=2,
            random_state=random_state,
            n_clusters_per_class=1,
            flip_y=0.001,
            class_sep=2.0,
        )
        rng = np.random.RandomState(random_state)
        X += 2 * rng.uniform(size=X.shape)
    elif dataset == "blobs":
        X, y = make_blobs(n_samples=n_samples, centers=5, random_state=random_state)
        y[y == 2] = 0
        y[y == 3] = 1
        y[y == 4] = 0
    else:
        X, y = make_moons(n_samples=n_samples, noise=0.2, random_state=random_state)
    X = MinMaxScaler().fit_transform(X)
    return X, y


X, y = simulate_data(dataset)
n_classes = int(y.max() + 1)
n_samples_train = X.shape[0]


@st.cache
def build_mesh(X):
    return get_mesh(X, h=h, padding=0.2)


xx, yy, X_mesh = build_mesh(X)


@st.cache
def get_decision(kernel, C):
    clf = SVC(kernel=kernel, C=C, degree=degree, gamma=gamma)
    clf.fit(X, y)
    zz = clf.decision_function(np.c_[xx.ravel(), yy.ravel()]).reshape(xx.shape)
    return zz, clf.support_vectors_


zz, sv = get_decision(kernel, C)
X_current = X
y_current = y


_ = plt.figure(figsize=(5, 4))
ax = plt.subplot(1, 1, 1)

plot_contour_binary_classif(ax, xx, yy, zz, levels=levels, norm=norm, support_vectors=sv)
if show_data:
    plot_scatter_binary_classif(ax, xx, yy, X_current, y_current, s=5, lw=1, norm=norm)
plt.legend(fontsize=8)
plt.tight_layout()
st.pyplot()


"""This small demo illustrates the influence of the hyperparameters in the Kernel SVM for binary classification.
"""
